package org.jtheque.books.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.dao.able.IDaoAuthors;
import org.jtheque.books.persistence.od.AuthorImpl;
import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.primary.dao.able.IDaoCountries;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * A DAO Authors implementation.
 *
 * @author Baptiste Wicht
 */
public final class DaoAuthors extends GenericDao<AuthorImpl> implements IDaoAuthors {
    private final ParameterizedRowMapper<AuthorImpl> rowMapper = new AuthorRowMapper();
    private final QueryMapper queryMapper = new AuthorQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private IDaoCountries daoCountries;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new DaoAuthors.
     */
    public DaoAuthors() {
        super(TABLE);
    }

    @Override
    public List<AuthorImpl> getAuthors() {
        return getAll();
    }

    @Override
    public AuthorImpl getAuthor(int id) {
        return get(id);
    }

    @Override
    public AuthorImpl getAuthor(String firstName, String name) {
        List<AuthorImpl> authors = jdbcTemplate.query("SELECT * FROM " + TABLE + " WHERE NAME = ? AND FIRSTNAME = ?", rowMapper, name, firstName);

        if(authors.isEmpty()){
            return null;
        }
        
        AuthorImpl author = authors.get(0);
        
        if (isNotInCache(author.getId())) {
            getCache().put(author.getId(), author);
        }

        return getCache().get(author.getId());
    }

    @Override
    public boolean exists(String firstName, String name) {
        return getAuthor(firstName, name) != null;
    }

    @Override
    protected ParameterizedRowMapper<AuthorImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<AuthorImpl> authors = persistenceContext.getSortedList(TABLE, rowMapper);

        for (AuthorImpl author : authors) {
            getCache().put(author.getId(), author);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        AuthorImpl author = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, author);
    }

    /**
     * A mapper to map a resultset to an author.
     *
     * @author Baptiste Wicht
     */
    private final class AuthorRowMapper implements ParameterizedRowMapper<AuthorImpl> {
        @Override
        public AuthorImpl mapRow(ResultSet rs, int i) throws SQLException {
            AuthorImpl author = new AuthorImpl();

            author.setId(rs.getInt("ID"));
            author.setName(rs.getString("NAME"));
            author.setFirstName(rs.getString("FIRSTNAME"));
            author.setTheCountry(daoCountries.getCountry(rs.getInt("THE_COUNTRY_FK")));
            author.setNote(DaoNotes.getINSTANCE().getNote(NoteType.getEnum(rs.getInt("NOTE"))));

            return author;
        }
    }

    /**
     * A mapper to map an author to a query.
     *
     * @author Baptiste Wicht
     */
    private static final class AuthorQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            AuthorImpl author = (AuthorImpl) entity;

            String query = "INSERT INTO " + TABLE + " (NAME, FIRSTNAME, THE_COUNTRY_FK, NOTE) VALUES(?,?,?,?)";

            Object[] parameters = {
                    author.getName(),
                    author.getFirstName(),
                    author.getTheCountry().getId(),
                    author.getNote().getValue().intValue()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            AuthorImpl author = (AuthorImpl) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ?, FIRSTNAME = ?, THE_COUNTRY_FK = ?, NOTE = ? WHERE ID = ?";

            Object[] parameters = {
                    author.getName(),
                    author.getFirstName(),
                    author.getTheCountry().getId(),
                    author.getNote().getValue().intValue(),
                    author.getId()};

            return new Query(query, parameters);
        }
    }
}