package org.jtheque.books.services.impl.utils.web.analyzers;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.BooksModule;
import org.jtheque.books.persistence.od.AuthorImpl;
import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.books.services.able.IEditorsService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.books.services.able.INotesService;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.StringUtils;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.utils.web.analyzers.generic.field.FieldGetter;
import org.jtheque.primary.utils.web.analyzers.generic.GenericGenerator;
import org.jtheque.primary.utils.web.analyzers.generic.operation.ScannerPossessor;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * A generic book analyzer. It seems an analyzer who takes its parser information from an XML file.
 *
 * @author Baptiste Wicht
 */
public final class GenericBookAnalyzer extends AbstractBookAnalyzer implements ScannerPossessor {
    @Resource
    private IKindsService kindsService;

    @Resource
    private IEditorsService editorsService;

    @Resource
    private INotesService notesService;

    @Resource
    private ICountriesService countriesService;

    @Resource
    private IAuthorsService authorsService;

    @Resource
    private BooksModule booksModule;
    
    /**
     * The generator of the field getters.
     */
    private final GenericGenerator generator;

    private FieldGetter dateGetter;
    private FieldGetter pagesGetter;
    private FieldGetter kindGetter;
    private FieldGetter editorGetter;
    private FieldGetter resumeGetter;
    private FieldGetter authorsGetter;
    private FieldGetter isbn10Getter;
    private FieldGetter isbn13Getter;

    /**
     * Construct a new GenericBookAnalyzer from a XML file.
     *
     * @param file The XML file.
     */
    public GenericBookAnalyzer(final String file) {
        super();

        generator = new GenericGenerator(this);
        generator.generate("org/jtheque/books/ressources/analyzers/" + file);

        init();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    /**
     * Init the parser.
     */
    private void init() {
        dateGetter = generator.getFieldGetter("date");
        pagesGetter = generator.getFieldGetter("pages");
        kindGetter = generator.getFieldGetter("kind");
        editorGetter = generator.getFieldGetter("editor");
        resumeGetter = generator.getFieldGetter("resume");
        authorsGetter = generator.getFieldGetter("authors");
        isbn10Getter = generator.getFieldGetter("isbn10");
        isbn13Getter = generator.getFieldGetter("isbn13");
    }

    @Override
    public void findDate(final String line) {
        if (isDateDo()) {
            return;
        }

        if (dateGetter.mustGet(line)) {
            String current = dateGetter.performOperations(line, this);

            String value = dateGetter.getValue(current);

            if (value != null) {
                getBook().setYear(Integer.parseInt(value));

                setDate(true);
            }
        }
    }

    @Override
    public void findPages(final String line) {
        if (isDurationDo()) {
            return;
        }

        if (pagesGetter.mustGet(line)) {
            String current = pagesGetter.performOperations(line, this);

            String value = pagesGetter.getValue(current);

            if (value != null) {
                int pages = Integer.parseInt(value);

                getBook().setPages(pages);

                setPages(true);
            }
        }
    }

    @Override
    public void findKind(final String line) {
        if (isKindDo()) {
            return;
        }

        if (kindGetter.mustGet(line)) {
            String current = kindGetter.performOperations(line, this);

            String value = kindGetter.getValue(current);

            if (value != null) {
                value = StringUtils.setFirstLetterUpper(value);

                if (kindsService.exists(value)) {
                    getBook().setTheKind(kindsService.getKind(value));
                } else {
                    KindImpl kind = new KindImpl();

                    kind.setName(value);

                    kindsService.create(kind);

                    getBook().setTheKind(kind);
                }

                setKind(true);
            }
        }
    }

    @Override
    public void findEditor(final String line) {
        if (isRealizerDo()) {
            return;
        }

        if (editorGetter.mustGet(line)) {
            String current = editorGetter.performOperations(line, this);

            String value = editorGetter.getValue(current);

            if (value != null) {
                if (editorsService.exists(value)) {
                    getBook().setTheEditor(editorsService.getEditor(value));
                } else {
                    EditorImpl editor = new EditorImpl();
                    editor.setName(value);

                    editorsService.create(editor);

                    getBook().setTheEditor(editor);
                }

                setEditor(true);
            }
        }
    }

    @Override
    public void findAuthors(String line) {
        if (isActorsDo()) {
            return;
        }

        if (authorsGetter.mustGet(line)) {
            String current = authorsGetter.performOperations(line, this);

            String value = authorsGetter.getValue(current);

            if (value != null) {
                List<AuthorImpl> authors = new ArrayList<AuthorImpl>(5);

                value = StringUtils.removeHTMLEntities(value);

                //On dcoupe la chaine en autant qu'il y a d'acteurs dedans
                String[] actorsTemp = value.split("\\+");

                //On parcours la listes des acteurs
                for (String name : actorsTemp) {
                    //On spare le nom en parties
                    String[] nomPrenom = name.split(" ");

                    StringBuilder nom = new StringBuilder(50);
                    String prenom = nomPrenom[0];

                    //On spare le nom et le prnom du ralisateur
                    for (int i = 1; i < nomPrenom.length; ++i) {
                        nom.append(StringUtils.setFirstLetterUpper(nomPrenom[i]));
                        nom.append(' ');
                    }

                    StringUtils.removeLastSpace(nom);

                    //Si cet acteur n'existe pas, on le cre
                    if (authorsService.exists(prenom, nom.toString())) {
                        AuthorImpl author = authorsService.getAuthor(prenom, nom.toString());
                        authors.add(author);
                    } else {
                        AuthorImpl author = new AuthorImpl();
                        author.setName(nom.toString());
                        author.setFirstName(prenom);
                        author.setTheCountry(countriesService.getDefaultCountry());
                        author.setNote(notesService.getDefaultNote());

                        authorsService.create(author);

                        authors.add(author);
                    }

                    if (authors.size() >= booksModule.getConfiguration().getNumberOfAuthors()) {
                        break;
                    }
                }

                getBook().setAuthors(authors);
            }
        }
    }

    @Override
    public void findResume(final String line) {
        if (isResumeDo()) {
            return;
        }

        if (resumeGetter.mustGet(line)) {
            String current = resumeGetter.performOperations(line, this);

            String value = resumeGetter.getValue(current);

            if (value != null) {
                value = value.replaceAll("\\+", "\n");
                getBook().setResume(StringUtils.removeHTMLEntities(value));

                setResume(true);
            }
        }
    }

    @Override
    public void findIsbn10(final String line) {
        if (isResumeDo()) {
            return;
        }

        if (isbn10Getter.mustGet(line)) {
            String current = isbn10Getter.performOperations(line, this);

            String value = isbn10Getter.getValue(current);

            if (value != null) {
                getBook().setIsbn10(value);

                setIsbn10(true);
            }
        }
    }

    @Override
    public void findIsbn13(final String line) {
        if (isResumeDo()) {
            return;
        }

        if (isbn13Getter.mustGet(line)) {
            String current = isbn13Getter.performOperations(line, this);

            String value = isbn13Getter.getValue(current);

            if (value != null) {
                getBook().setIsbn13(value);

                setIsbn10(true);
            }
        }
    }

    @Override
    public Scanner getScanner() {
        return null;
    }
}