package org.jtheque.books.view.models;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.BookImpl;
import org.jtheque.books.services.able.IBooksService;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.listeners.ViewStateListener;
import org.jtheque.primary.view.impl.models.PrincipalDataModel;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * A principal data model for the books view.
 *
 * @author Baptiste Wicht
 */
public final class BooksModel extends PrincipalDataModel {
    private boolean enabled;
    private BookImpl currentBook;

    private List<BookImpl> displayList;

    @Resource
    private IBooksService booksService;

    /**
     * Init the model.
     */
    @PostConstruct
    private void init() {
        booksService.addDataListener(this);
    }

    @Override
    @SuppressWarnings("unchecked")
    public void updateDisplayList(List<? extends Entity> datas) {
        displayList.clear();

        Collection<BookImpl> books = (Collection<BookImpl>) datas;

        if (books == null) {
            displayList.addAll(booksService.getBooks());
        } else {
            displayList.addAll(books);
        }

        fireDisplayListChanged();
    }

    @Override
    public void updateDisplayList() {
        updateDisplayList(null);
    }

    /**
     * Return the display list.
     *
     * @return The display list
     */
    public List<BookImpl> getDisplayList() {
        if (displayList == null) {
            displayList = new ArrayList<BookImpl>(25);
            updateDisplayList();
        }

        return displayList;
    }

    /**
     * Enable or disable the view.
     *
     * @param enabled true if the view has to be enabled else false.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;

        fireStateChanged();
    }

    /**
     * Indicate if the view is enabled or not.
     *
     * @return true if the view is enabled else false.
     */
    public boolean isEnabled() {
        return enabled;
    }

    /**
     * Return the current book.
     *
     * @return The current book.
     */
    public BookImpl getCurrentBook() {
        return currentBook;
    }

    /**
     * Set the current book.
     *
     * @param currentBook The new current book
     */
    public void setCurrentBook(BookImpl currentBook) {
        this.currentBook = currentBook;

        fireCurrentObjectChanged(new ObjectChangedEvent(this, currentBook));
    }

    /**
     * Add a view state listener.
     *
     * @param listener The listener to add.
     */
    public void addViewStateListener(ViewStateListener listener) {
        getEventListenerList().add(ViewStateListener.class, listener);
    }

    /**
     * Remove a view state listener.
     *
     * @param listener The listener to remove.
     */
    public void removeViewStateListener(ViewStateListener listener) {
        getEventListenerList().remove(ViewStateListener.class, listener);
    }

    /**
     * Fire a event who indicate that the current state has changed.
     */
    private void fireStateChanged() {
        ViewStateListener[] listeners = getEventListenerList().getListeners(ViewStateListener.class);

        for (ViewStateListener listener : listeners) {
            listener.stateChanged();
        }
    }
}